﻿using e_commerceProject.Models;
using e_commerceProject.Services.PasswordHasher;
using e_commerceProject.Services.UserService;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;


namespace e_commerceProject.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class RegistrationController : ControllerBase
    {
        private readonly IUserService _userService;
        private readonly IPasswordHasher _passwordHasher;
        private readonly IConfiguration _configuration;

        public RegistrationController(IUserService userService, IPasswordHasher passwordHasher, IConfiguration configuration)
        {
            _userService = userService;
            _passwordHasher = passwordHasher;   
            _configuration = configuration;
        }

        [HttpGet, Authorize(Roles = "Admin")]
        public async Task<ActionResult<List<UserModel>>> GetAllUsers()
        {
            try
            {
                return await _userService.GetAllUsers();
            }
            catch(Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, $"Error while creating an order: {ex.Message}");
            }

        }

        [HttpGet("{id}")]
        public async Task<ActionResult<UserModel>> GetSingleUser(int id)
        {
            try
            {
                var result = await _userService.GetSingleUser(id);
                if (result is null)
                    return StatusCode(StatusCodes.Status404NotFound, "User not found.");

                return StatusCode(StatusCodes.Status200OK, result);
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, $"Error while getting a user: {ex.Message}");
            }
        }

        [HttpPost("register")]
        public async Task<ActionResult<List<UserModel>>> AddUser(UserRegisterDTO user)
        {
            try
            {
                var result = await _userService.CreateUser(user);
                return StatusCode(StatusCodes.Status200OK, result);
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, $"Error while creating an user: {ex.Message}");
            }
        }

        [HttpPost("login")]
        public async Task<ActionResult<UserModel>> Login(UserLoginDTO request)
        {
            try
            {
                if (string.IsNullOrEmpty(request.Username))
                    return StatusCode(StatusCodes.Status400BadRequest, "You have to specify a username.");

                var user = await _userService.GetSingleUserByUsername(request.Username);

                if (user != null)
                {

                    if (!_passwordHasher.Verify(request.Password, user.Password))
                        return StatusCode(StatusCodes.Status400BadRequest, "Wrong password.");

                    string token = CreateToken(user);

                    Response.Headers.Add("Authorization", "Bearer " + token);

                    return Ok(new { token });

                }

                return StatusCode(StatusCodes.Status400BadRequest, "User not found.");
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, $"Error while login: {ex.Message}");
            }


        }

        private string CreateToken(UserModel user)
        {
            var claims = new List<Claim> {
                   new Claim(ClaimTypes.Name, user.Username),
                   new Claim(ClaimTypes.Role, user.Role)
            };

            var securityKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(this._configuration.GetSection("Jwt:Key").Value!));
            var credentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha512);
           
            var token = new JwtSecurityToken(
                issuer: this._configuration.GetSection("Jwt:Issuer").Value!,
                audience: this._configuration.GetSection("Jwt:Audience").Value!,
                claims: claims, 
                expires: DateTime.UtcNow.AddMinutes(1),
                signingCredentials: credentials
            );
            return new JwtSecurityTokenHandler().WriteToken(token);
        }

        [HttpGet("GetUserName"), Authorize]
        public ActionResult<string> GetMyName()
        {
            try
            {
                return Ok(_userService.GetMyNameAsync());
            }
            catch (Exception ex)
            {
                return StatusCode(StatusCodes.Status500InternalServerError, $"Error: {ex.Message}");
            }

        }


    }
}
