﻿using e_commerceProject.Models;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Http;
using System.Security.Claims;
using Microsoft.EntityFrameworkCore;
using e_commerceProject.Services.UserService;
using e_commerceProject.Services.OrderStatusService;
using e_commerceProject.Services.ProductService;

namespace e_commerceProject.Services.OrderService
{
    public class OrderService : IOrderService
    {
        private readonly ApplicationDbContext _context;
        private readonly IUserService _userService;
        private readonly IOrderStatusService _orderStatusService;
        private readonly IProductService _productService;

        public OrderService( ApplicationDbContext context, IUserService userService, IOrderStatusService orderStatusService, IProductService productService)
        {
            _context = context;
            _userService = userService;
            _orderStatusService = orderStatusService;
            _productService = productService;
        }

        public async Task<OrderModel> AddOrderAsync(OrderCreateDTO order)
        {
            var username = await _userService.GetMyNameAsync();
            var user = await _userService.GetSingleUserByUsername(username);
            var newOrder = new OrderModel
            {
                User = user,
                OrderStatus = await _orderStatusService.GetSingleStatus(1),
                OrderDetails = new List<OrderDetail>()
            };

            var totalCost = 0;
            var selectedProductIds = order.OrderDetails.Select(od => od.ProductId).ToList();
            var selectedProducts = await _productService.GetAllProductsWithIds(selectedProductIds);

            foreach (var orderDetailsDTO in order.OrderDetails)
            {
                var product = GetProduct(selectedProducts, orderDetailsDTO.ProductId);
                CheckProductAvailability(product, orderDetailsDTO.Quantity);

                var newOrderDetail = CreateNewOrderDetail(product, orderDetailsDTO.Quantity);
                UpdateProductStock(product, orderDetailsDTO.Quantity);

                totalCost += CalculateOrderDetailCost(orderDetailsDTO.Quantity, product.Price);
                newOrder.OrderDetails.Add(newOrderDetail);
            }

            newOrder.TotalCost = totalCost;

            _context.Orders.Add(newOrder);
            await _context.SaveChangesAsync();

            return newOrder;
        }

        private ProductModel GetProduct(List<ProductModel> products, int productId)
        {
            var product = products.FirstOrDefault(p => p.Id == productId);
            if (product == null)
            {
                throw new Exception($"Product with ID '{productId}' not found.");
            }
            return product;
        }

        private void CheckProductAvailability(ProductModel product, int quantity)
        {
            if (!int.TryParse(product.Stock, out int productStock))
            {
                throw new Exception($"Incorrect inventory value for the product '{product.Name}'.");
            }
            if (productStock < quantity)
            {
                throw new Exception($"Product '{product.Name}' not available in requested quantity.");
            }
        }

        private OrderDetail CreateNewOrderDetail(ProductModel product, int quantity)
        {
            return new OrderDetail
            {
                Product = product,
                Quantity = quantity
            };
        }

        private void UpdateProductStock(ProductModel product, int quantity)
        {
            int productStock = int.Parse(product.Stock);
            product.Stock = (productStock - quantity).ToString();
        }

        private int CalculateOrderDetailCost(int quantity, string price)
        {
            return quantity * int.Parse(price);
        }

        public async Task<string> GetOrderStatus(int orderId)
        {
                var order = await _context.Orders
                    .Include(o => o.OrderStatus)
                    .FirstOrDefaultAsync(o => o.Id == orderId);

                if (order == null)
                {
                    return "Order not found.";
                }

                return $"Order status for you OrderId: {orderId} is: {order.OrderStatus.StatusName}";
            
        }

    }
}
